@@ -47,6 +47,9 @@
 #include "dri2.h"
 #include "dri_common.h"
 
+#undef DRI2_MINOR
+#define DRI2_MINOR 1
+
 typedef struct __GLXDRIdisplayPrivateRec __GLXDRIdisplayPrivate;
 typedef struct __GLXDRIcontextPrivateRec __GLXDRIcontextPrivate;
 typedef struct __GLXDRIdrawablePrivateRec __GLXDRIdrawablePrivate;
@@ -73,6 +76,8 @@ struct __GLXDRIdrawablePrivateRec {
     __DRIbuffer buffers[5];
     int bufferCount;
     int width, height;
+    int have_back;
+    int have_fake_front;
 };
 
 static void dri2DestroyContext(__GLXDRIcontext *context,
@@ -166,6 +171,7 @@ static __GLXDRIdrawable *dri2CreateDrawable(__GLXscreenConfigs *psc,
     pdraw->base.xDrawable = xDrawable;
     pdraw->base.drawable = drawable;
     pdraw->base.psc = psc;
+    pdraw->bufferCount = 0;
 
     DRI2CreateDrawable(psc->dpy, xDrawable);
 
@@ -190,12 +196,22 @@ static void dri2CopySubBuffer(__GLXDRIdrawable *pdraw,
     XRectangle xrect;
     XserverRegion region;
 
+    /* Check we have the right attachments */
+    if (!priv->have_back)
+    	return;
+
     xrect.x = x;
     xrect.y = priv->height - y - height;
     xrect.width = width;
     xrect.height = height;
 
+#ifdef __DRI2_FLUSH
+    if (pdraw->psc->f)
+    	(*pdraw->psc->f->flush)(pdraw->driDrawable);
+#endif
+
     region = XFixesCreateRegion(pdraw->psc->dpy, &xrect, 1);
+    /* should get a fence ID back from here at some point */
     DRI2CopyRegion(pdraw->psc->dpy, pdraw->drawable, region,
 		   DRI2BufferFrontLeft, DRI2BufferBackLeft);
     XFixesDestroyRegion(pdraw->psc->dpy, region);
@@ -208,6 +224,66 @@ static void dri2SwapBuffers(__GLXDRIdrawable *pdraw)
     dri2CopySubBuffer(pdraw, 0, 0, priv->width, priv->height);
 }
 
+static void dri2WaitX(__GLXDRIdrawable *pdraw)
+{
+    __GLXDRIdrawablePrivate *priv = (__GLXDRIdrawablePrivate *) pdraw;
+    XRectangle xrect;
+    XserverRegion region;
+
+    /* Check we have the right attachments */
+    if (!priv->have_fake_front)
+    	return;
+
+    xrect.x = 0;
+    xrect.y = 0;
+    xrect.width = priv->width;
+    xrect.height = priv->height;
+
+#ifdef __DRI2_FLUSH
+    if (pdraw->psc->f)
+    	(*pdraw->psc->f->flush)(pdraw->driDrawable);
+#endif
+
+    region = XFixesCreateRegion(pdraw->psc->dpy, &xrect, 1);
+    DRI2CopyRegion(pdraw->psc->dpy, pdraw->drawable, region,
+		   DRI2BufferFakeFrontLeft, DRI2BufferFrontLeft);
+    XFixesDestroyRegion(pdraw->psc->dpy, region);
+}
+
+static void dri2WaitGL(__GLXDRIdrawable *pdraw)
+{
+    __GLXDRIdrawablePrivate *priv = (__GLXDRIdrawablePrivate *) pdraw;
+    XRectangle xrect;
+    XserverRegion region;
+
+    if (!priv->have_fake_front)
+    	return;
+
+    xrect.x = 0;
+    xrect.y = 0;
+    xrect.width = priv->width;
+    xrect.height = priv->height;
+
+#ifdef __DRI2_FLUSH
+    if (pdraw->psc->f)
+    	(*pdraw->psc->f->flush)(pdraw->driDrawable);
+#endif
+
+    region = XFixesCreateRegion(pdraw->psc->dpy, &xrect, 1);
+    DRI2CopyRegion(pdraw->psc->dpy, pdraw->drawable, region,
+		   DRI2BufferFrontLeft, DRI2BufferFakeFrontLeft);
+    XFixesDestroyRegion(pdraw->psc->dpy, region);
+}
+
+
+static void dri2FlushFrontBuffer(__DRIdrawable *driDrawable,
+				 void *loaderPrivate)
+{
+    (void) driDrawable;
+    dri2WaitGL((__GLXDRIdrawable *) loaderPrivate);
+}
+
+
 static void dri2DestroyScreen(__GLXscreenConfigs *psc)
 {
     /* Free the direct rendering per screen data */
@@ -216,6 +292,38 @@ static void dri2DestroyScreen(__GLXscreenConfigs *psc)
     psc->__driScreen = NULL;
 }
 
+/**
+ * Process list of buffer received from the server
+ *
+ * Processes the list of buffers received in a reply from the server to either
+ * \c DRI2GetBuffers or \c DRI2GetBuffersWithFormat.
+ */
+static void
+process_buffers(__GLXDRIdrawablePrivate *pdraw, DRI2Buffer *buffers,
+		unsigned count)
+{
+    int i;
+
+    pdraw->bufferCount = count;
+    pdraw->have_fake_front = 0;
+    pdraw->have_back = 0;
+
+    /* This assumes the DRI2 buffer attachment tokens matches the
+     * __DRIbuffer tokens. */
+    for (i = 0; i < count; i++) {
+	pdraw->buffers[i].attachment = buffers[i].attachment;
+	pdraw->buffers[i].name = buffers[i].name;
+	pdraw->buffers[i].pitch = buffers[i].pitch;
+	pdraw->buffers[i].cpp = buffers[i].cpp;
+	pdraw->buffers[i].flags = buffers[i].flags;
+	if (pdraw->buffers[i].attachment == __DRI_BUFFER_FAKE_FRONT_LEFT)
+	    pdraw->have_fake_front = 1;
+	if (pdraw->buffers[i].attachment == __DRI_BUFFER_BACK_LEFT)
+	    pdraw->have_back = 1;
+    }
+
+}
+
 static __DRIbuffer *
 dri2GetBuffers(__DRIdrawable *driDrawable,
 	       int *width, int *height,
@@ -224,7 +332,6 @@ dri2GetBuffers(__DRIdrawable *driDrawable,
 {
     __GLXDRIdrawablePrivate *pdraw = loaderPrivate;
     DRI2Buffer *buffers;
-    int i;
 
     buffers = DRI2GetBuffers(pdraw->base.psc->dpy, pdraw->base.xDrawable,
 			     width, height, attachments, count, out_count);
@@ -233,16 +340,32 @@ dri2GetBuffers(__DRIdrawable *driDrawable,
 
     pdraw->width = *width;
     pdraw->height = *height;
+    process_buffers(pdraw, buffers, *out_count);
 
-    /* This assumes the DRI2 buffer attachment tokens matches the
-     * __DRIbuffer tokens. */
-    for (i = 0; i < *out_count; i++) {
-	pdraw->buffers[i].attachment = buffers[i].attachment;
-	pdraw->buffers[i].name = buffers[i].name;
-	pdraw->buffers[i].pitch = buffers[i].pitch;
-	pdraw->buffers[i].cpp = buffers[i].cpp;
-	pdraw->buffers[i].flags = buffers[i].flags;
-    }
+    Xfree(buffers);
+
+    return pdraw->buffers;
+}
+
+static __DRIbuffer *
+dri2GetBuffersWithFormat(__DRIdrawable *driDrawable,
+			 int *width, int *height,
+			 unsigned int *attachments, int count,
+			 int *out_count, void *loaderPrivate)
+{
+    __GLXDRIdrawablePrivate *pdraw = loaderPrivate;
+    DRI2Buffer *buffers;
+
+    buffers = DRI2GetBuffersWithFormat(pdraw->base.psc->dpy,
+				       pdraw->base.xDrawable,
+				       width, height, attachments,
+				       count, out_count);
+    if (buffers == NULL)
+       return NULL;
+
+    pdraw->width = *width;
+    pdraw->height = *height;
+    process_buffers(pdraw, buffers, *out_count);
 
     Xfree(buffers);
 
@@ -252,6 +375,15 @@ dri2GetBuffers(__DRIdrawable *driDrawable,
 static const __DRIdri2LoaderExtension dri2LoaderExtension = {
     { __DRI_DRI2_LOADER, __DRI_DRI2_LOADER_VERSION },
     dri2GetBuffers,
+    dri2FlushFrontBuffer,
+    dri2GetBuffersWithFormat,
+};
+
+static const __DRIdri2LoaderExtension dri2LoaderExtension_old = {
+    { __DRI_DRI2_LOADER, __DRI_DRI2_LOADER_VERSION },
+    dri2GetBuffers,
+    dri2FlushFrontBuffer,
+    NULL,
 };
 
 static const __DRIextension *loader_extensions[] = {
@@ -260,11 +392,19 @@ static const __DRIextension *loader_extensions[] = {
     NULL
 };
 
+static const __DRIextension *loader_extensions_old[] = {
+    &dri2LoaderExtension_old.base,
+    &systemTimeExtension.base,
+    NULL
+};
+
 static __GLXDRIscreen *dri2CreateScreen(__GLXscreenConfigs *psc, int screen,
 					__GLXdisplayPrivate *priv)
 {
     const __DRIconfig **driver_configs;
     const __DRIextension **extensions;
+    const __GLXDRIdisplayPrivate *const pdp = (__GLXDRIdisplayPrivate *)
+      priv->dri2Display;
     __GLXDRIscreen *psp;
     char *driverName, *deviceName;
     drm_magic_t magic;
@@ -282,8 +422,10 @@ static __GLXDRIscreen *dri2CreateScreen(__GLXscreenConfigs *psc, int screen,
 	return NULL;
 
     psc->driver = driOpenDriver(driverName);
-    if (psc->driver == NULL)
+    if (psc->driver == NULL) {
+	ErrorMessageF("driver pointer missing\n");
 	goto handle_error;
+    }
 
     extensions = dlsym(psc->driver, __DRI_DRIVER_EXTENSIONS);
     if (extensions == NULL) {
@@ -309,15 +451,26 @@ static __GLXDRIscreen *dri2CreateScreen(__GLXscreenConfigs *psc, int screen,
 	return NULL;
     }
 
-    if (drmGetMagic(psc->fd, &magic))
+    if (drmGetMagic(psc->fd, &magic)) {
+	ErrorMessageF("failed to get magic\n");
 	return NULL;
+    }
 
-    if (!DRI2Authenticate(psc->dpy, RootWindow(psc->dpy, screen), magic))
+    if (!DRI2Authenticate(psc->dpy, RootWindow(psc->dpy, screen), magic)) {
+	ErrorMessageF("failed to authenticate magic %d\n", magic);
 	return NULL;
+    }
 
+    /* If the server does not support the protocol for
+     * DRI2GetBuffersWithFormat, don't supply that interface to the driver.
+     */
     psc->__driScreen = 
-	psc->dri2->createNewScreen(screen, psc->fd,
-				   loader_extensions, &driver_configs, psc);
+      psc->dri2->createNewScreen(screen, psc->fd,
+				 ((pdp->driMinor < 1)
+				  ? loader_extensions_old
+				  : loader_extensions),
+				 &driver_configs, psc);
+
     if (psc->__driScreen == NULL) {
 	ErrorMessageF("failed to create dri screen\n");
 	return NULL;
@@ -328,10 +481,14 @@ static __GLXDRIscreen *dri2CreateScreen(__GLXscreenConfigs *psc, int screen,
     psc->configs = driConvertConfigs(psc->core, psc->configs, driver_configs);
     psc->visuals = driConvertConfigs(psc->core, psc->visuals, driver_configs);
 
+    psc->driver_configs = driver_configs;
+
     psp->destroyScreen = dri2DestroyScreen;
     psp->createContext = dri2CreateContext;
     psp->createDrawable = dri2CreateDrawable;
     psp->swapBuffers = dri2SwapBuffers;
+    psp->waitGL = dri2WaitGL;
+    psp->waitX = dri2WaitX;
 
     /* DRI2 suports SubBuffer through DRI2CopyRegion, so it's always
      * available.*/
