@@ -30,14 +30,13 @@
 
 
 #include "glheader.h"
-#include "glapi.h"
-#include "glapitable.h"
 #include "macros.h"
-#include "colormac.h"
 #include "api_loopback.h"
-#include "glthread.h"
 #include "mtypes.h"
-#include "dispatch.h"
+#include "glapi/glapi.h"
+#include "glapi/glapitable.h"
+#include "glapi/glthread.h"
+#include "glapi/dispatch.h"
 
 /* KW: A set of functions to convert unusual Color/Normal/Vertex/etc
  * calls to a smaller set of driver-provided formats.  Currently just
@@ -146,7 +145,7 @@ static void GLAPIENTRY
 loopback_Color3iv_f( const GLint *v )
 {
    COLORF( INT_TO_FLOAT(v[0]), INT_TO_FLOAT(v[1]),
-	   INT_TO_FLOAT(v[2]), INT_TO_FLOAT(v[3]) );
+	   INT_TO_FLOAT(v[2]), 1.0 );
 }
 
 static void GLAPIENTRY
