@@ -85,7 +85,7 @@ lookup_statevar(const char *var, GLint index1, GLint index2, const char *field,
       { "gl_TextureMatrixTranspose", STATE_TEXTURE_MATRIX, 0 },
       { "gl_TextureMatrixInverseTranspose", STATE_TEXTURE_MATRIX, STATE_MATRIX_INVERSE },
 
-      { "gl_NormalMatrix", STATE_MODELVIEW_MATRIX, STATE_MATRIX_TRANSPOSE },
+      { "gl_NormalMatrix", STATE_MODELVIEW_MATRIX, STATE_MATRIX_INVERSE },
 
       { NULL, 0, 0 }
    };
@@ -111,10 +111,9 @@ lookup_statevar(const char *var, GLint index1, GLint index2, const char *field,
 
    if (isMatrix) {
       if (tokens[0] == STATE_TEXTURE_MATRIX) {
-         if (index1 >= 0) {
-            tokens[1] = index1; /* which texture matrix */
-            index1 = 0; /* prevent extra addition at end of function */
-         }
+         /* texture_matrix[index1][index2] */
+         tokens[1] = index1 >= 0 ? index1 : 0; /* which texture matrix */
+         index1 = index2; /* move matrix row value to index1 */
       }
       if (index1 < 0) {
          /* index1 is unused: prevent extra addition at end of function */
@@ -437,7 +436,7 @@ emit_statevars(const char *name, int array_len,
                struct gl_program_parameter_list *paramList)
 {
    if (type->type == SLANG_SPEC_ARRAY) {
-      GLint i, pos;
+      GLint i, pos = -1;
       assert(array_len > 0);
       if (strcmp(name, "gl_ClipPlane") == 0) {
          tokens[0] = STATE_CLIPPLANE;
@@ -502,7 +501,7 @@ emit_statevars(const char *name, int array_len,
    }
    else if (type->type == SLANG_SPEC_STRUCT) {
       const slang_variable_scope *fields = type->_struct->fields;
-      GLuint i, pos;
+      GLuint i, pos = 0;
       for (i = 0; i < fields->num_variables; i++) {
          const slang_variable *var = fields->variables[i];
          GLint p = emit_statevars(var->a_name, 0, &var->type.specifier,
@@ -682,7 +681,9 @@ _slang_alloc_statevar(slang_ir_node *n,
    if (n->Opcode == IR_ELEMENT) {
       /* XXX can only handle constant indexes for now */
       if (n->Children[1]->Opcode == IR_FLOAT) {
-         index2 = (GLint) n->Children[1]->Value[0];
+         /* two-dimensional array index: mat[i][j] */
+         index2 = index1;
+         index1 = (GLint) n->Children[1]->Value[0];
       }
       else {
          *direct = GL_FALSE;
